(* CVT2FONT.PAS -- dump GEOS fonts
** Copyright (C) 1995,1996  Jochen Metzinger
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

PROGRAM cvt2font;

USES global, fonts, cvt;

TYPE
 FontHeader =
  RECORD
    ul: BYTE; (* Pixels minus 1 above the underline. *)
   bss: WORD; (* Bytes in the bit stream *)
   pnt: BYTE; (* Point size, character height in pixels *)
   idx: WORD; (* Index to table of bit stream indices *)
    bs: WORD; (* Index to first bit stream. *)
  END;

 FontIndices =
  ARRAY [32..127] OF WORD;
  (* Indices for char ' ' to '~' *)

 FontBitStream =
  ARRAY [0..BufferMaxSize] OF BYTE;
  (* Bit stream of font *)

VAR
 font_id: STRING;
 info: InfoRec;
 x0: INTEGER;
 chain: INTEGER;
 fheader: ^FontHeader;
 findex: ^FontIndices;
 fstream: ^FontBitStream;
 lines: ARRAY [0..125] OF STRING;
 out_file: TEXT;

PROCEDURE usage;
BEGIN
 WriteLn('SHOWFONT V', Version, Country, '  Convert GEOS font files');
 WriteLn('Copyright (c) 1995,1996 Jochen Metzinger ');
 WriteLn;
 WriteLn(short_usage);
 WriteLn;
 WriteLn('  filename  GEOS file');
 WriteLn('  output    dump to output');
 HALT(1);
END; (* usage *)

PROCEDURE ClearLines;
 VAR i: INTEGER;
BEGIN
 FOR i := 0 TO 125 DO
  lines[i] := '';
END; (* ClearLines *)

PROCEDURE PrintLines(nr: INTEGER);
 VAR i: INTEGER;
BEGIN
 FOR i := 0 TO nr-1 DO BEGIN
  WriteLn(out_file, lines[i]);
  lines[i] := '';
 END; (* for *)
 WriteLn(out_file);
END; (* PrintLines *)

PROCEDURE OutLine(y: INTEGER; p: WORD; b: BYTE; width: WORD);
 VAR c: BYTE;
BEGIN
 c := (fstream^[p] SHL b) AND $FF;
 WHILE width > 0 DO BEGIN
  IF c AND $80 <> 0
   THEN lines[y] := lines[y] + '*'
   ELSE lines[y] := lines[y] + '.';
  INC(b);
  IF b > 7 THEN
   BEGIN
    b := 0;
    INC(p);
    c := fstream^[p];
   END
  ELSE
   c := (c SHL 1) AND $FF;
  DEC(width);
 END; (* while *)
END; (* OutLine *)

PROCEDURE OutChar(fsize: INTEGER; c: INTEGER);
 VAR pos: WORD; width: LongInt;
  row: BYTE;
BEGIN
 pos := findex^[c];
 width := (findex^[c+1] - pos) - 1;
 IF width < 0 THEN
  FATAL('font '+long2str(fsize)+' at "'+CHR(c)+'" negativ width');
 IF pos DIV 8 > fheader^.bss THEN
  FATAL('font '+long2str(fsize)+' at "'+CHR(c)+'" index out of range');
 IF x0 + width >= 78 THEN BEGIN
  PrintLines(fsize);
  x0 := 0;
 END; (* if *)
 FOR row := 0 TO fsize-1 DO BEGIN
  OutLine(row, fheader^.bss*row+(pos DIV 8), pos MOD 8, width);
  lines[row] := lines[row] + ' ';
 END; (* for *)
 x0 := x0 + width + 1;
END; (* OutChar *)

PROCEDURE OutFont(nr: INTEGER);
 VAR i: INTEGER;
BEGIN
 cvt_chain(nr);
 IF cvt_size(nr) < SizeOf(FontHeader) THEN
  FATAL('font '+long2str(nr)+' too short');
 fheader := POINTER(Buffer);
 WITH fheader^ DO BEGIN
  IF pnt <> nr THEN
   FATAL('font '+long2str(nr)+' has wrong chain');
  IF ul > pnt THEN
   FATAL('font '+long2str(nr)+' underline error');
  IF SizeOf(fontHeader) > idx THEN
   FATAL('font '+long2str(nr)+' header & index crash');
  IF idx+SizeOf(FontIndices) > bs THEN
   FATAL('font '+long2str(nr)+' index & stream crash');
  IF bs + pnt*bss > cvt_size(nr) THEN
   FATAL('font '+long2str(nr)+' stream incomplete');
  findex  := Addr(Buffer^[idx]);
  fstream := Addr(Buffer^[bs]);
 END; (* with *)
 WriteLn(out_file, '#');
 WriteLn(out_file, '# ',font_id,' ',nr,',',fheader^.ul);
 WriteLn(out_file, '#');
 WriteLn(out_file);
 x0 := 0;
 FOR i := 32 TO 126 DO
  OutChar(nr,i);
 PrintLines(nr);
 WriteLn(out_file);
 x0 := 0;
END; (* OutFont *)

BEGIN
 short_usage := 'SHOWFONT [/?] filename [output]';
 IF (ParamStr(1) = '/?') OR (ParamStr(1) = '') THEN usage;
 cvt_open(ParamStr(1));
 font_id := cvt_name;
 cvt_info(info);
 IF info.gtyp <> 8 THEN FATAL('not a FONT');
 font_id := font_id + '['+long2str(info.w[128 DIV 2])+']';
 ClearLines;
 {$I-}
 Assign(out_file, ParamStr(2));
 ReWrite(out_file);
 IF IOResult <> 0 THEN
  FATAL('unable to open output');
 {$I+}
 FOR chain := 0 TO 126 DO
  IF cvt_chain_used(chain) THEN
   OutFont(chain);
 cvt_close;
 Close(out_file);
END. (* cvt2font *)
