(* CVT2PCX.PAS -- translate GEOS graphic files
** Copyright (C) 1995,1996  Jochen Metzinger
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

PROGRAM cvt2pcx;

USES DOS,
 global, cvt, paint, photo;

PROCEDURE usage;
BEGIN
 WriteLn('CVT2PCX V', Version, '  Convert GEOS graphic files');
 WriteLn('Copyright (c) 1995,1996 Jochen Metzinger ');
 WriteLn;
 WriteLn(short_usage);
 WriteLn;
 WriteLn('  input     GEOS file (geoPaint, geoWrite, photo scrap/album)');
 WriteLn('  output    PCX output file');
 WriteLn('  /1        stop on error');
 WriteLn('  /2        continue on error');
 WriteLn('  /3        black/white on error');
 WriteLn('  /4        interactive correction of errors');
 WriteLn('  /Q        quiet translation');
 WriteLn('  /V        only view');
 HALT(1);
END; (* usage *)

VAR out_name: STRING;
 level: WORD;
 view: BOOLEAN;

PROCEDURE Init;
 VAR i, j: WORD; par: STRING;
  in_name: STRING; do_pcx: BOOLEAN;
BEGIN
 short_usage := 'CVT2PCX [/?] input [output] [options]';
 in_name := ''; out_name := '';
 level := 4;
 view := TRUE; do_pcx := TRUE;
 IF ParamCount = 0 THEN
  usage;
 FOR i := 1 TO ParamCount DO BEGIN
  par := ParamStr(i);
  IF Copy(par,1,1) = '/' THEN
   BEGIN
    IF Length(par) = 1 THEN
     FATAL('unknown option '+par);
    FOR j := 2 TO Length(par) DO
     CASE UpCase(par[j]) OF
      '1': level := 1;
      '2': level := 2;
      '3': level := 3;
      '4': level := 4;
      'Q': BEGIN  view := FALSE; do_pcx := TRUE; END;
      'V': BEGIN  view := TRUE; do_pcx := FALSE; END;
      '?': usage;
      ELSE FATAL('unknown option '+par);
     END; (* case *)
   END
  ELSE IF in_name = '' THEN
   in_name := par
  ELSE IF out_name = '' THEN
   out_name := par
  ELSE
   FATAL('too many arguments');
 END; (* for *)
 IF in_name = '' THEN usage;
 cvt_open(in_name);
 IF do_pcx AND (out_name = '') THEN
  out_name := GetFileName(in_name);
 short_usage := '';
END; (* init *)

FUNCTION ChainPCX(fn: STRING; chain: ShortInt): STRING;
(* return name for chain *)
 VAR D: DirStr; N: NameStr; E: ExtStr;
BEGIN
 IF fn = '' THEN
  ChainPCX := ''
 ELSE BEGIN
  FSplit(fn, D, N, E);
  IF Length(N) > 5 THEN
   Delete(N, 6, Length(N)-5);
  N := N + '$' + HexStr(chain,2);
  ChainPCX := D + N + E;
 END; (* else *)
END; (* ChainPCX *)

VAR
 info: InfoRec;
 chain, min: ShortInt;
 empty: BOOLEAN;

BEGIN
 init;
 cvt_info(info);
 IF IsPaint(info) THEN
  DoPaint(out_name, view, level)
 ELSE IF IsPhotoScrap(info) THEN
  BEGIN
   OpenScrap(view, level);
   DoScrap(0, out_name);
   CloseScrap;
  END
 ELSE
  BEGIN
   IF IsPhotoAlbum(info) THEN
    min := 0
   ELSE IF IsWrite(info) THEN
    min := 64
   ELSE
    FATAL('not a graphic file');
   empty := TRUE;
   FOR chain := min TO 126 DO
    IF cvt_chain_used(chain) THEN
     empty := FALSE;
   IF empty THEN
    FATAL('no scraps in document');
   OpenScrap(view, level);
   FOR chain := min TO 126 DO
    IF cvt_chain_used(chain) THEN
     DoScrap(chain, ChainPCX(out_name,chain));
   CloseScrap;
  END; (* else *)
 cvt_close;
END. (* cvt2pcx *)
