(* D64.PAS -- access C=1541 image
** Copyright (C) 1995,1996  Jochen Metzinger
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

UNIT D64;

INTERFACE

USES global;

CONST
 BLOCK_SIZE  = 256; (* size of a C=64 block *)
 D64_SIZE    = 683; (* blocks on a disk *)

TYPE
 D64_BLOCK = ARRAY [0..BLOCK_SIZE-1] OF BYTE;

VAR
 DiskName, DiskID: STRING;
 (* Name and ID of disk *)
 BorderTr, BorderSc: BYTE;
 (* geos: Track/Sector for DESKTOP's buffer *)

PROCEDURE D64_open(fname: STRING; geos: BOOLEAN);
(* open (GEOS) image file *)
PROCEDURE D64_close;
(* close it *)
PROCEDURE D64_read(tr, sk: BYTE; VAR bl: D64_BLOCK);
(* read block at tr/sk *)
PROCEDURE D64_write(tr, sk: BYTE; VAR bl: D64_BLOCK);
(* write block to tr/sk *)

IMPLEMENTATION

USES errors;

TYPE
 REGION = RECORD von, bis, snr: BYTE; offs: WORD; END;

CONST
 geometry: ARRAY [1..4] OF REGION =
  ((von: 01; bis: 17; snr: 21; offs: 000),
   (von: 18; bis: 24; snr: 19; offs: 357),
   (von: 25; bis: 30; snr: 18; offs: 490),
   (von: 31; bis: 35; snr: 17; offs: 598));

VAR
 D64_disk: FILE OF D64_BLOCK;

PROCEDURE D64_open(fname: STRING; geos: BOOLEAN);
 VAR BAM: D64_BLOCK;
  i: INTEGER; geos_magic: STRING;
BEGIN
 (*$I-*)
 Assign(D64_disk, AddExt(fname, '.D64'));
 Reset(D64_disk);
 (*$I+*)
 IF IOResult <> 0 THEN
  FATAL(E_NOFILE);
 IF FileSize(D64_disk) <> D64_SIZE THEN
  FATAL(E_IMAGE);
 D64_read(18,0,BAM);
 IF is_err THEN FATAL(errno);
 IF (BAM[$02] <> $41) OR (BAM[$A5] <> $32) OR (BAM[$A6] <> $41) THEN
  FATAL(E_IMAGE);
 DiskName := GetString(BAM[$90],16);
 i := Pos(#$A0,DiskName);
 IF i > 0 THEN Delete(DiskName,i,$FF);
 DiskID := GetString(BAM[$A2],2);
 IF geos THEN BEGIN
  (* is it a GEOS image? *)
  geos_magic := GetString(BAM[$AD],11);
  IF geos_magic <> 'GEOS format' THEN
   FATAL(E_GEOS);
  BorderTr := BAM[$AB];
  BorderSc := BAM[$AC];
 END; (* if *)
END; (* D64_open *)

PROCEDURE D64_close;
BEGIN
 Close(D64_disk);
END; (* D64_close *)

PROCEDURE D64_seek(tr, sk: BYTE; et: ErrorType);
 VAR i: BYTE; pos: WORD;
BEGIN
 error(E_NONE);
 err_p1 := tr; err_p2 := sk;
 IF tr = 0 THEN BEGIN
  error(E_TRKSEC);
  EXIT;
 END; (* if *)
 FOR i := 1 TO 4 DO
  IF tr <= geometry[i].bis THEN BEGIN
   IF sk >= geometry[i].snr THEN BEGIN
    error(E_TRKSEC);
    EXIT;
   END; (* if *)
   WITH geometry[i] DO
    pos := offs + snr*(tr - von) + sk;
   (*$I-*)
   Seek(D64_disk,pos);
   (*$I+*)
   IF IOResult <> 0 THEN error(et);
   EXIT;
  END; (* if *)
 error(E_TRKSEC);
END; (* D64_seek *)

PROCEDURE D64_read(tr, sk: BYTE; VAR bl: D64_BLOCK);
BEGIN
 D64_seek(tr,sk,E_READ);
 IF is_err THEN EXIT;
 (*$I-*)
 Read(D64_disk,bl);
 (*$I+*)
 IF IOResult <> 0 THEN FATAL(E_READ);
END; (* D64_read *)

PROCEDURE D64_write(tr, sk: BYTE; VAR bl: D64_BLOCK);
BEGIN
 D64_seek(tr,sk,E_WRITE);
 IF is_err THEN EXIT;
 (*$I-*)
 Write(D64_disk,bl);
 (*$I+*)
 IF IOResult <> 0 THEN FATAL(E_WRITE);
END; (* D64_write *)

BEGIN
 DiskName := ''; DiskID := '';
 BorderTr := 0; BorderSc := 0;
END. (* D64 *)