(* ERRORS.PAS -- handle errors
** Copyright (C) 1995,1996  Jochen Metzinger
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

UNIT errors;

INTERFACE

TYPE
 ErrorType =
  (E_NONE,
   E_NOFILE, E_IMAGE, E_TRKSEC, E_READ, E_WRITE, E_GEOS, (* D64 *)
   E_PARAM, E_MANYARG, E_CREAT, E_STRUCT, E_LARGE, E_TAIL (* GDIR/GGET *)
  );

VAR
 errno: ErrorType;
 (* error state *)
 err_p1, err_p2: LongInt;
 (* numerical error arguments *)
 err_s: STRING;
 (* more error text *)
 short_usage: STRING;
 (* short description of program *)

PROCEDURE error(err: ErrorType);
(* set error condition *)
FUNCTION is_err: BOOLEAN;
(* test for an error *)
PROCEDURE err_recover;
(* print out error and continue *)
PROCEDURE FATAL(err: ErrorType);
(* print out error and stop program *)
PROCEDURE err_stop;
(* if is_err then FATAL() *)

IMPLEMENTATION

PROCEDURE error(err: ErrorType);
BEGIN
 errno := err;
END; (* error *)

FUNCTION is_err: BOOLEAN;
BEGIN
  is_err := errno <> E_NONE;
END; (* is_err *)

PROCEDURE out_error(typ: STRING);
BEGIN
 IF errno = E_NONE THEN EXIT;
 Write(typ,': ');
 CASE errno OF
  E_NOFILE: WriteLn('file not found');
  E_IMAGE: WriteLn('not a C=1541 image');
  E_TRKSEC: WriteLn('illegal track and sector ',err_p1,' ',err_p2);
  E_READ: WriteLn('read file');
  E_WRITE: WriteLn('write file');
  E_GEOS: WriteLn('not a GEOS disk image');
  E_PARAM: WriteLn('unknown option ',err_s);
  E_MANYARG: WriteLn('too many arguments');
  E_CREAT: WriteLn('unable to create file');
  E_STRUCT: WriteLn('not SEQ nor VLIR');
  E_LARGE: WriteLn('chain/file too large');
  E_TAIL: WriteLn('tail sector');
  ELSE WriteLn('unknown ',ORD(errno));
 END;
END; (* out_error *)

PROCEDURE err_recover;
(* gibt den Fehlertext aus *)
BEGIN
 out_error('ERROR');
 errno := E_NONE;
END; (* err_recover *)

PROCEDURE FATAL(err: ErrorType);
BEGIN
 errno := err;
 out_error('FATAL');
 IF short_usage <> '' THEN
  WriteLn(short_usage);
 HALT(1);
END; (* FATAL *)

PROCEDURE err_stop;
BEGIN
 IF is_err THEN FATAL(errno);
END; (* err_stop *)

BEGIN
 errno := E_NONE;
 short_usage := '';
END. (* errors *)
