(* FONTS.PAS -- translate from GEOS fonts to IBM code page 437
** Copyright (C) 1995,1996  Jochen Metzinger
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

UNIT Fonts;

INTERFACE

USES global;

CONST
{$IFDEF GERMAN}
 DefaultFont = $080; (* BSW_GE *)
 Country     = 'ge';
{$ELSE}
 DefaultFont = $000; (* BSW *)
 Country     = 'uk';
{$ENDIF}

FUNCTION TransStr(s: STRING): STRING;
(* Translate s in a PC string *)

FUNCTION TransChr(ch: CHAR): STRING;
(* Translate ch in a PC string *)

PROCEDURE SelectFont(id: WORD);
(* select translation of a font *)

FUNCTION FontName: STRING;
(* return current font name *)

IMPLEMENTATION

CONST
 NR_CODING =  2;
 NR_FONT   = 35;

TYPE
 FontInfo   = RECORD id, code: WORD; name: STRING[16]; END;
 CodingType = ARRAY [' '..'~'] OF CHAR;

CONST
 font: ARRAY [1..NR_FONT] OF FontInfo =
  ((id: $000; code: 0; name: 'BSW'),
   (id: $001; code: 0; name: 'University'),
   (id: $002; code: 0; name: 'California'),
   (id: $003; code: 0; name: 'Roma'),
   (id: $004; code: 0; name: 'Dwinelle'),
   (id: $005; code: 0; name: 'Cory'),
   (id: $006; code: 0; name: 'Tolman'),
   (id: $007; code: 0; name: 'Bubble'),
   (id: $008; code: 0; name: 'Fontknox'),
   (id: $009; code: 0; name: 'Harmon'),
   (id: $00A; code: 0; name: 'Mykonos'),
   (id: $00B; code: 0; name: 'Boalt'),
   (id: $00C; code: 0; name: 'Stadium'),
   (id: $00D; code: 0; name: 'Tilden'),
   (id: $00E; code: 0; name: 'Evans'),
   (id: $00F; code: 0; name: 'Durant'),
   (id: $010; code: 0; name: 'Telegraph'),
   (id: $011; code: 0; name: 'Superb'),
   (id: $012; code: 0; name: 'Bowditch'),
   (id: $013; code: 0; name: 'Ormond'),
   (id: $014; code: 0; name: 'Elmwood'),
   (id: $015; code: 0; name: 'Hearst'),
   (id: $016; code: 0; name: 'Brennens'),
   (id: $017; code: 0; name: 'Channing'),
   (id: $018; code: 0; name: 'Putnam'),
   (id: $019; code: 0; name: 'LeConte'),
   (id: $038; code: 1; name: 'University_GE'),
   (id: $039; code: 1; name: 'California_GE'),
   (id: $040; code: 1; name: 'Roma_GE'),
   (id: $041; code: 1; name: 'Dwinelle_GE'),
   (id: $042; code: 1; name: 'Cory_GE'),
   (id: $058; code: 1; name: 'LW_Roma_GE'),
   (id: $059; code: 1; name: 'LW_Cal_GE'),
   (id: $05B; code: 1; name: 'LW_Barrows_GE'),
   (id: $080; code: 1; name: 'BSW_GE'));

 Coding: ARRAY [0..NR_CODING-1] OF CodingType =
  ((* 0 = english *)
   (' ','!','"','#','$','%','&','''','(',')','*','+',',','-','.','/',
    '0','1','2','3','4','5','6','7','8','9',':',';','<','=','>','?',
    '@','A','B','C','D','E','F','G','H','I','J','K','L','M','N','O',
    'P','Q','R','S','T','U','V','W','X','Y','Z','[','\',']','^','_',
    '`','a','b','c','d','e','f','g','h','i','j','k','l','m','n','o',
    'p','q','r','s','t','u','v','w','x','y','z','{','|','}','~'),
   (* 1 = german *)
   (' ','!','"','#','$','%','&','''','(',')','*','+',',','-','.','/',
    '0','1','2','3','4','5','6','7','8','9',':',';','<','=','>','?',
    '','A','B','C','D','E','F','G','H','I','J','K','L','M','N','O',
    'P','Q','R','S','T','U','V','W','X','Y','Z','','','','^','_',
    '`','a','b','c','d','e','f','g','h','i','j','k','l','m','n','o',
    'p','q','r','s','t','u','v','w','x','y','z','','','',''));

VAR
 FontID, FontCoding: WORD;
 _FontName: STRING;

FUNCTION TransChr(ch: CHAR): STRING;
BEGIN
 CASE ch OF
  #$00..#$1F: TransChr := '^^' + CHR(ORD(ch)+64);
  #$20..#$7E: TransChr := Coding[FontCoding,ch];
        #$7F: TransChr := '^^?';
  #$80..#$FF: TransChr := '{'+long2str(ORD(ch))+'}';
 END; (* case *)
END; (* TransChr *)

FUNCTION TransStr(s: STRING): STRING;
 VAR i: INTEGER; r: STRING;
BEGIN
 r := '';
 FOR i := 1 TO Length(s) DO
  r := r + TransChr(s[i]);
 TransStr := r;
END; (* TransStr *)

PROCEDURE SelectFont(id: WORD);
 VAR i: INTEGER;
BEGIN
 FOR i := 1 TO NR_FONT DO
  IF font[i].id = id THEN BEGIN
   FontID := font[i].id;
   FontCoding := font[i].code;
   _FontName := font[i].name;
   EXIT;
  END; (* if *)
 FOR i := 1 TO NR_FONT DO
  IF font[i].id = DefaultFont THEN BEGIN
   FontID := id;
   FontCoding := font[i].code;
   _FontName := '#'+long2str(id)+'#';
   EXIT;
  END; (* if *)
 WriteLn('FONTS.TPU: no default font');
 HALT(3);
END; (* SelectFont *)

FUNCTION FontName: STRING;
BEGIN
 FontName := _FontName;
END; (* FontName *)

BEGIN
 SelectFont(DefaultFont);
END. (* Fonts *)
