(* GDIR.PAS -- show directory of a GEOS disk image
** Copyright (C) 1995,1996  Jochen Metzinger
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

PROGRAM Gdir;

USES
 global, fonts, errors, D64, dir;

CONST
 DosTypeName: ARRAY [$0..$7] OF STRING[3] =
  ('DEL', 'SEQ', 'PRG', 'USR',
   'REL', '?05', '?06', '?07');

 GeosTypeName: ARRAY [0..15] OF STRING[4] =
  ('C=64', 'BAS ', 'ASM ', 'DATA',
   'SYS ', 'HELP', 'APPL', 'DOC ',
   'FONT', 'PRT ', 'IN  ', 'DISK',
   'STRT', 'TMP ', 'SELF', 'MOUS');

VAR
 f_name: STRING;
 use_dir: ARRAY [-1..9] OF BOOLEAN;

PROCEDURE usage;
BEGIN
 WriteLn('GDIR V', Version, Country, '  View directory of a GEOS disk image');
 WriteLn('Copyright (c) 1995,1996 Jochen Metzinger ');
 WriteLn;
 WriteLn(short_usage);
 WriteLn;
 WriteLn('  image       disk image');
 WriteLn('  filename    show this file(s)');
 WriteLn('  /1 .. /9    view this page of the directory');
 WriteLn('  /B          view border');
 HALT(1);
END; (* usage *)

PROCEDURE Init;
 VAR i, j: INTEGER; par: STRING;
  i_name: STRING; all_dir: BOOLEAN;
BEGIN
 short_usage := 'GDIR [/?] image [filename] [options]';
 f_name := '';
 FOR i := -1 TO 9 DO
  use_dir[i] := FALSE;
 i_name := '';
 all_dir := TRUE;
 IF ParamCount = 0 THEN
  usage;
 FOR i := 1 TO ParamCount DO BEGIN
  par := ParamStr(i); err_s := par;
  IF Copy(par,1,1) = '/' THEN
   BEGIN
    IF Length(par) = 1 THEN
     FATAL(E_PARAM);
    FOR j := 2 TO Length(par) DO
     CASE UpCase(par[j]) OF
      '?': usage;
      '0'..'9': BEGIN
        all_dir := FALSE;
        use_dir[ORD(par[i])-ORD('0')] := TRUE;
       END;
      'B': BEGIN
        all_dir := FALSE;
        use_dir[0] := TRUE;
       END;
      ELSE FATAL(E_PARAM);
     END; (* case *)
   END
  ELSE IF i_name = '' THEN
   i_name := par
  ELSE IF f_name = '' THEN
   BEGIN
    i := Pos('*',par);
    IF i > 0 THEN Delete(par,i+1,$FF);
    f_name := Copy(par,1,16);
   END
  ELSE
   FATAL(E_MANYARG);
 END; (* for *)
 IF i_name = '' THEN usage;
 IF f_name = '' THEN f_name := '*';
 IF all_dir THEN
  FOR i := -1 TO 9 DO
   use_dir[i] := TRUE;
 D64_open(i_name,TRUE);
 short_usage := '';
END; (* init *)

PROCEDURE Write02ui(d: WORD);
BEGIN
 Write((d DIV 10) MOD 10);
 Write(d MOD 10);
END; (* Write02ui *)

PROCEDURE WriteDirRec(d: DirRec);
 VAR fn: STRING;
BEGIN
 WITH d DO BEGIN
  Write(size:3);
  IF dostype AND $80 <> 0 THEN Write(' ') ELSE Write('*');
  fn := FileName(d);
  Write('"',fn,'"':17-Length(fn));
  IF dostype AND $40 <> 0 THEN Write('<') ELSE Write(' ');
  Write(DosTypeName[dostype AND $07],'   ');
  IF geostype > 15 THEN
   Write('??',HexStr(geostype,2),' ')
  ELSE
   Write(GeosTypeName[geostype],' ');
  IF geostype <> 0 THEN
   BEGIN (* only GEOS files has structure and time stamp *)
    IF struct = 0 THEN
     Write('SEQ    ')
    ELSE IF struct = 1 THEN
     Write('VLIR   ')
    ELSE
     Write('??',HexStr(struct,2),'   ');
    Write(1900+year,'-');
    Write02ui(month); Write('-');
    Write02ui(day); Write(' ');
    Write02ui(hour); Write(':');
    Write02ui(minute);
   END; (* if *)
  WriteLn;
 END; (* with *)
END; (* WriteDirRec *)

FUNCTION OutPageP(nr: INTEGER): BOOLEAN;
BEGIN
 OutPageP := TRUE;
 IF use_dir[-1] THEN EXIT;
 OutPageP := FALSE;
 IF (nr < 0) OR (nr > 9) THEN EXIT;
 OutPageP := use_dir[nr];
END; (* OutPageP *)

FUNCTION LIKE(fn, pt: STRING): BOOLEAN;
 VAR i: INTEGER;
BEGIN
 LIKE := FALSE;
 FOR i := 1 TO Length(pt) DO BEGIN
  IF pt[i] = '*' THEN BEGIN
   LIKE := TRUE;
   EXIT;
  END; (* if *)
  IF Length(fn) < i THEN
   EXIT;
  IF pt[i] <> '?' THEN
   IF pt[i] <> fn[i] THEN
    EXIT;
 END; (* for *)
 LIKE := (Length(fn) = Length(pt));
END; (* LIKE *)

PROCEDURE OutputDir;
VAR dr: DirRec; nr: INTEGER;
 first, found: BOOLEAN;
BEGIN (* OutputDir *)
 nr := -1;
 first := FALSE;
 found := FALSE;
 OpenDir;
 WHILE NOT EndOfDir DO BEGIN
  IF FirstDirPage THEN BEGIN
   Inc(nr);
   first := TRUE;
  END; (* if *)
  ReadDir(dr);
  IF dr.dostype <> 0 THEN
   IF OutPageP(nr) THEN
    IF LIKE(FileName(dr), f_name) THEN BEGIN
     IF first THEN BEGIN
      WriteLn;
      WriteLn(' *** directory ',nr,' ***');
      first := FALSE;
     END; (* if *)
     WriteDirRec(dr);
     found := TRUE;
    END; (* if *)
 END; (* while *)
 CloseDir;
 IF NOT found THEN BEGIN
  WriteLn;
  WriteLn('no file');
 END; (* if *)
END; (* OutputDir *)

BEGIN
 init;
 WriteLn;
 WriteLn(' Speichermedium ist ',TransStr(DiskName),',',TransStr(DiskID));
 OutputDir;
 d64_close;
END. (* Gdir *)
