(* GGET.PAS -- Get files from a GEOS disk image
** Copyright (C) 1995,1996  Jochen Metzinger
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

PROGRAM Gget;

USES Crt,
 global, fonts, errors, D64, dir;

CONST
 FILE_CHARS: SET OF CHAR =
  ['!','#'..')','-','0'..'9','@'..'Z','^'..'{','}','~'];
 DosTypeName: ARRAY [$0..$7] OF STRING[4] =
  ('DEL', 'SEQ', 'PRG', 'USR',
   'REL', 'X05', 'X06', 'X07');

VAR
 f_name: STRING;
 use_dir: ARRAY [-1..9] OF BOOLEAN;
 interactiv, force: BOOLEAN;

PROCEDURE usage;
BEGIN
 WriteLn('GGET V', Version, Country, '  Get files from a GEOS disk image');
 WriteLn('Copyright (c) 1995,1996 Jochen Metzinger ');
 WriteLn;
 WriteLn(short_usage);
 WriteLn;
 WriteLn('  image       disk image');
 WriteLn('  filename    get this file(s)');
 WriteLn('  /1 .. /9    from this page of the directory');
 WriteLn('  /B          from border');
 WriteLn('  /I          ask for each file');
 WriteLn('  /O          don''t ask if file exist');
 HALT(1);
END; (* usage *)

PROCEDURE Init;
 VAR i, j: INTEGER; par: STRING;
  i_name: STRING; all_dir: BOOLEAN;
BEGIN
 short_usage := 'GGET [/?] image filename [options]';
 f_name := '';
 FOR i := -1 TO 9 DO use_dir[i] := FALSE;
 interactiv := FALSE;
 force := FALSE;
 i_name := '';
 all_dir := TRUE;
 IF ParamCount = 0 THEN
  usage;
 FOR i := 1 TO ParamCount DO BEGIN
  par := ParamStr(i); err_s := par;
  IF Copy(par,1,1) = '/' THEN
   BEGIN
    IF Length(par) = 1 THEN
     FATAL(E_PARAM);
    FOR j := 2 TO Length(par) DO
     CASE UpCase(par[j]) OF
      '?': usage;
      '0'..'9': BEGIN
        all_dir := FALSE;
        use_dir[ORD(par[j])-ORD('0')] := TRUE;
       END;
      'B': BEGIN
        all_dir := FALSE;
        use_dir[0] := TRUE;
       END;
      'I': interactiv := TRUE;
      'O': force := TRUE;
      ELSE FATAL(E_PARAM);
     END; (* case *)
   END
  ELSE IF i_name = '' THEN
   i_name := par
  ELSE IF f_name = '' THEN
   BEGIN
    j := Pos('*',par);
    IF j > 0 THEN Delete(par,j+1,$FF);
    f_name := Copy(par,1,16);
   END
  ELSE
   FATAL(E_MANYARG);
 END; (* for *)
 IF i_name = '' THEN usage;
 IF f_name = '' THEN f_name := '*';
 IF all_dir THEN
  FOR i := -1 TO 9 DO
   use_dir[i] := TRUE;
 D64_open(i_name,TRUE);
 short_usage := '';
END; (* init *)

FUNCTION MakeDosFileName(gname, ext: STRING): STRING;
(* convert GEOS file names *)
 VAR fn: STRING; i: BYTE;
BEGIN
 fn := Copy(gname,1,8);
 FOR i := 1 TO Length(fn) DO
  IF fn[i] IN FILE_CHARS THEN
   fn[i] := UpCase(fn[i])
  ELSE IF fn[i] = ' ' THEN
   fn[i] := '_'
  ELSE
   fn[i] := 'X';
 MakeDosFileName := fn + '.' + ext;
END; (* MakeDosFileName *)

FUNCTION FileExists(FileName: STRING): BOOLEAN;
 VAR f: FILE;
BEGIN
 (*$I-*)
 Assign(f, FileName);
 Reset(f);
 Close(f);
 (*$I+*)
 FileExists := (IOResult = 0) and (FileName <> '');
END; (* FileExists *)

FUNCTION GetFileHandler(VAR f: FILE; gname, ext: STRING): BOOLEAN;
 VAR fn: STRING; ch, dummy: CHAR;
BEGIN
 GetFileHandler := FALSE;
 fn := MakeDosFileName(gname, ext);
 Write(' -> ',fn);
 IF interactiv THEN BEGIN
  Write(' (Y,n)? ');
  REPEAT
   ch := UpCase(ReadKey);
   IF ch = #0 THEN dummy := ReadKey;
   IF ch = 'N' THEN BEGIN
    WriteLn('N');
    EXIT;
   END; (* if *)
  UNTIL ch IN ['Y',#13];
  Write('Y');
 END; (* if *)
 WriteLn;
 IF NOT force THEN BEGIN
  ch := 'X';
  WHILE (ch <> 'Y') AND FileExists(fn) DO BEGIN
   Write('GGET: Warning! ', fn, ' already exists. Overwrite (y/n/a/r)? ');
   REPEAT
    ch := UpCase(ReadKey);
    IF ch = #0 THEN dummy := ReadKey;
   UNTIL ch IN ['Y','N','A','R'];
   WriteLn(ch);
   CASE ch OF
    'N': EXIT;
    'Y': (* nothing *);
    'A': BEGIN
      ch := 'Y';
      force := TRUE;
     END;
    'R': BEGIN
      Write('New name? ');
      ReadLn(fn);
     END;
   END; (* case *)
  END; (* while *)
 END; (* if *)
 {$I-}
 Assign(f, fn);
 ReWrite(f, 1);
 {$I+}
 IF IOResult <> 0
  THEN error(E_CREAT)
  ELSE GetFileHandler := TRUE;
END; (* GetFileHandler *)

FUNCTION CopySEQ(VAR t, s: BYTE; VAR f: FILE): WORD;
 VAR block: D64_BLOCK; cnt, res: WORD;
BEGIN
 CopySEQ := 0;
 cnt := 0;
 WHILE t <> 0 DO BEGIN
  d64_read(t,s,block);
  IF is_err THEN EXIT;
  t := block[0];
  s := block[1];
  BlockWrite(f, block[2], SizeOf(block)-2, res);
  IF res <> SizeOf(block)-2 THEN EXIT;
  INC(cnt);
  IF cnt > $FF THEN BEGIN
   ERROR(E_LARGE);
   EXIT;
  END; (* if *)
 END; (* while *)
 t := cnt;
 IF cnt > 0 THEN BEGIN
  IF s < 2 THEN BEGIN
   ERROR(E_TAIL);
   s := $FF;
  END; (* if *)
  CopySEQ := $FF - s;
 END; (* if *)
END; (* CopySEQ *)

PROCEDURE NoGeosFile(VAR dir: DirRec);
 VAR head: D64_BLOCK; magic: STRING;
  cvt_file: FILE; ext: STRING;
  back: LongInt;
BEGIN
 (* handle converted file *)
 ext := DosTypeName[dir.dostype AND $07];
 D64_read(dir.tr_1st, dir.sc_1st, head);
 IF is_err THEN BEGIN
  WriteLn;
  EXIT;
 END; (* if *)
 IF head[0] > 0 THEN
  BEGIN (* not last block *)
   magic := GetString(head[$20], 28);
   IF (ext = 'SEQ') OR (ext = 'PRG') THEN
    IF magic = ext + ' formatted GEOS file V1.0' THEN
     ext := CVT_EXT;
  END; (* if *)
 (* Transfer simple file *)
 IF NOT GetFileHandler(cvt_file,FileName(dir),ext) THEN EXIT;
 back := CopySEQ(dir.tr_1st, dir.sc_1st, cvt_file);
 IF back > 0 THEN BEGIN
  Seek(cvt_file, FileSize(cvt_file)-back);
  Truncate(cvt_file);
 END; (* if *)
 Close(cvt_file);
END; (* NoGeosFile *)

PROCEDURE TransferFile(dir: DirRec);
 VAR fn: STRING; cvt_file: FILE; res: WORD;
  head, info, vlir: D64_BLOCK;
  i: INTEGER; back: LongInt;
BEGIN
 fn := FileName(dir);
 Write('"',fn,'"':17-Length(fn));
 IF dir.geostype = 0 THEN BEGIN
  NoGeosFile(dir);
  EXIT;
 END; (* if *)
 IF dir.struct > 1 THEN BEGIN
  ERROR(E_STRUCT);
  WriteLn;
  EXIT;
 END; (* if *)
 IF NOT GetFileHandler(cvt_file,FileName(dir),CVT_EXT) THEN
  EXIT;
 (* Output header0 *)
 FillChar(head, SizeOf(head), 0);
 Move(dir, head[$02], SizeOf(dir));
 BlockWrite(cvt_file, head[2], SizeOf(head)-2, res);
 IF res <> SizeOf(head)-2 THEN FATAL(E_WRITE);
 (* Output info rec *)
 D64_read(dir.tr_info, dir.sc_info, info);
 IF is_err THEN EXIT;
 BlockWrite(cvt_file, info[2], SizeOf(info)-2, res);
 IF res <> SizeOf(info)-2 THEN FATAL(E_WRITE);
 IF dir.struct = 1 THEN
  BEGIN (* VLIR format *)
   (* Output vlir0 *)
   D64_read(dir.tr_1st, dir.sc_1st, vlir);
   IF is_err THEN EXIT;
   BlockWrite(cvt_file, vlir[2], SizeOf(vlir)-2, res);
   IF res <> SizeOf(vlir)-2 THEN FATAL(E_WRITE);
   (* Transfer chains *)
   back := 0;
   FOR i := 1 TO 127 DO
    IF vlir[2*i] <> 0 THEN BEGIN
     back := CopySEQ(vlir[2*i], vlir[2*i+1], cvt_file);
     IF is_err THEN EXIT;
    END; (* if *)
   (* Output vlir *)
   Seek(cvt_file, 2*(BLOCK_SIZE-2));
   BlockWrite(cvt_file, vlir[2], SizeOf(vlir)-2, res);
   IF res <> SizeOf(vlir)-2 THEN FATAL(E_WRITE);
  END
 ELSE
  BEGIN (* Sequential structure *)
   back := CopySEQ(dir.tr_1st, dir.sc_1st, cvt_file);
  END; (* else *)
 IF back > 0 THEN BEGIN
  Seek(cvt_file, FileSize(cvt_file)-back);
  Truncate(cvt_file);
 END; (* if *)
 (* Output header *)
 Seek(cvt_file,0);
 PutString(head[$20], 'SEQ formatted GEOS file V1.0');
 PutString(head[$A0], 'JOCHEN METZINGER''S GGET V'+Version);
 BlockWrite(cvt_file, head[2], SizeOf(head)-2, res);
 IF res <> SizeOf(head)-2 THEN FATAL(E_WRITE);
 Close(cvt_file);
END; (* TransferFile *)

FUNCTION OutPageP(nr: INTEGER): BOOLEAN;
BEGIN
 OutPageP := TRUE;
 IF use_dir[-1] THEN EXIT;
 OutPageP := FALSE;
 IF (nr < 0) OR (nr > 9) THEN EXIT;
 OutPageP := use_dir[nr];
END; (* OutPageP *)

FUNCTION LIKE(fn, pt: STRING): BOOLEAN;
 VAR i: INTEGER;
BEGIN
 LIKE := FALSE;
 FOR i := 1 TO Length(pt) DO BEGIN
  IF pt[i] = '*' THEN BEGIN
   LIKE := TRUE;
   EXIT;
  END; (* if *)
  IF Length(fn) < i THEN
   EXIT;
  IF pt[i] <> '?' THEN
   IF pt[i] <> fn[i] THEN
     EXIT;
 END; (* for *)
 LIKE := (Length(fn) = Length(pt));
END; (* LIKE *)

PROCEDURE DoTransfer;
VAR dir: DirRec;
 nr: INTEGER;
 found: BOOLEAN;
BEGIN
 nr := -1;
 found := FALSE;
 OpenDir;
 WHILE NOT EndOfDir DO BEGIN
  IF FirstDirPage THEN
   Inc(nr);
  ReadDir(dir);
  IF dir.dostype <> 0 THEN
   IF OutPageP(nr) THEN
    IF LIKE(FileName(dir),f_name) THEN BEGIN
     TransferFile(dir);
     IF is_err THEN err_recover;
     found := TRUE;
    END; (* if *)
 END; (* while *)
 CloseDir;
 IF NOT found THEN BEGIN
  WriteLn;
  WriteLn('no file');
 END; (* if *)
END; (* DoTransfer *)

BEGIN
 Init;
 WriteLn('Disk: ',TransStr(DiskName),',',TransStr(DiskID));
 DoTransfer;
 D64_close;
END. (* Gget *)
