(* GLOBAL.PAS -- global stuff
** Copyright (C) 1995,1996  Jochen Metzinger
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

UNIT global;

INTERFACE

USES Dos;

CONST
 CVT_EXT = 'CVT';
 Version = '0.1';

TYPE
 DirRec =
  RECORD
   dostype: BYTE;
   (* DOS file type *)
   tr_1st, sc_1st: BYTE;
   (* Track/Sector number of first sector *)
   name: ARRAY [1..16] OF CHAR;
   (* File name *)
   tr_info, sc_info: BYTE;
   (* Track/Sector number for info sector *)
   struct: BYTE;
   (* File structure *)
   geostype: BYTE;
   (* GEOS file type *)
   year, month, day, hour, minute: BYTE;
   (* time and  date  of  the  file's creation *)
   size: WORD;
   (* File size in blocks *)
  END; (* DirRec *)

 InfoRec = RECORD
   CASE INTEGER OF
    0: (b: ARRAY [2..255] OF BYTE);
    1: (w: ARRAY [1..127] OF WORD);
    2: (icon_8x, icon_y: BYTE;
     HiResMode: BYTE;
     icon: ARRAY [0..62] OF BYTE;
     ftyp: BYTE;
     gtyp: BYTE;
     strk: BYTE;
     prg_start: WORD;
     prg_end: WORD;
     sys_addr: WORD;
     class: ARRAY [1..20] OF CHAR;
     disk: ARRAY [1..20] OF CHAR;
     appl: ARRAY [1..20] OF CHAR;
     intern: ARRAY [1..23] OF BYTE;
     text: ARRAY [1..96] OF CHAR);
  END; (* InfoRec *)

FUNCTION long2str(l: LongInt): STRING;
(* LongInt -> STRING *)

FUNCTION HexStr(n: LongInt; b: BYTE): STRING;
(* LongInt -> STRING *)

FUNCTION GetString(VAR m; sz: BYTE): STRING;
(* return m[0..size-1] as string *)

PROCEDURE PutString(VAR m; s: STRING);
(* move s to m *)

FUNCTION GetFileName(path: STRING): STRING;
(* return file name w/o path,extension *)

FUNCTION AddExt(path, ext: STRING): STRING;
(* return path with extension *)

IMPLEMENTATION

FUNCTION long2str(l: LongInt): STRING;
 VAR s: STRING;
BEGIN
 Str(l,s);
 long2str := s;
END; (* long2str *)

FUNCTION HexStr(n: LongInt; b: BYTE): STRING;
 CONST hdigit : ARRAY [0..15] OF CHAR = '0123456789ABCDEF';
 VAR o: STRING;
BEGIN
 o := '';
 WHILE b > 0 DO BEGIN
  o := hdigit[n MOD 16] + o;
  n := n DIV 16;
  Dec(b);
 END;
 HexStr := o;
END; (* HexStr *)



FUNCTION GetString(VAR m; sz: BYTE): STRING;
 VAR s: STRING;
BEGIN
 s[0] := CHR(sz);
 Move(m, s[1], sz);
 GetString := s;
END; (* GetString *)

PROCEDURE PutString(VAR m; s: STRING);
BEGIN
 Move(s[1], m, ORD(s[0]));
END; (* PutString *)

FUNCTION GetFileName(path: STRING): STRING;
 VAR D: DirStr; N: NameStr; E: ExtStr;
BEGIN
 FSplit(path, D, N, E);
 GetFileName := N;
END; (* GetFileName *)

FUNCTION AddExt(path, ext: STRING): STRING;
 VAR D: DirStr; N: NameStr; E: ExtStr;
BEGIN
 FSplit(path, D, N, E);
 IF E = '' THEN E := ext;
 AddExt := D + N + E;
END; (* AddExt *)

END. (* global *)
