(* PAINT.PAS -- translate geoPaint graphics
** Copyright (C) 1995,1996  Jochen Metzinger
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

UNIT Paint;

INTERFACE

USES Crt, Graph,
 global, graphic, cvt, pcx;

CONST
 BW_COLOR = $BF; (* dark gray on lt grey *)

FUNCTION IsPaint(VAR info: InfoRec): BOOLEAN;
(* is it a geoPaint document *)

PROCEDURE DoPaint(pcx_name: STRING; view: BOOLEAN; lvl: WORD);
(* do translation *)

IMPLEMENTATION

CONST
 PatSize = 2*80*8;
 NulSize = 8;
 ColSize = 2*80;
 FrmSize = PatSize + NulSize + ColSize;
 ErrSize = 2*Patsize;
 MaxSize = ErrSize + 1024;

TYPE
 GeosImage = RECORD
  sz: WORD;
  CASE INTEGER OF
   0: (raw: ARRAY [0..MaxSize]   OF BYTE);
   1: (pat: ARRAY [0..1, 0..79, 0..7] OF BYTE;
       gap: ARRAY [0..NulSize-1] OF BYTE;
       col: ARRAY [0..1, 0..79] OF BYTE);
   2: (err: ARRAY [0..3, 0..79, 0..7] OF BYTE);
  END; (* GeosImage *)
 ImageType = RECORD
  x, y: INTEGER;
  p: ARRAY [0..31, 0..3, 0..79] OF BYTE;
  __res: WORD;
  END; (* ImageType *)

VAR
 chain: BYTE;
  (* used chain *)
 geos_image: ^GeosImage;
  (* Paint part / GEOS *)
 BitMap: ^ImageType;
  (* Paint part / EGA-Image *)
 err_fp, err_ip: LongInt; err_str: STRING;
  (* Errors from GetGeosImage() *)
 y: LongInt;
  (* for output with scrolling *)
 do_view: BOOLEAN;
  (* graphical output? *)
 do_pcx: BOOLEAN;
  (* create .PCX ? *)
 err_level: WORD;
  (* level of error recovery *)

FUNCTION IsPaint(VAR info: InfoRec): BOOLEAN;
BEGIN
 IsPaint := FALSE;
 WITH info DO BEGIN
  (* USR *)
  IF ftyp AND $07 <> $03 THEN EXIT;
  (* DOC *)
  IF gtyp <> $07 THEN EXIT;
  (* VLIR *)
  IF strk <> 1 THEN EXIT;
  (* Class: 'Paint Image V1.0 *)
  IF GetString(class,15) <> 'Paint Image V1.' THEN EXIT;
 END; (* with *)
 IsPaint := TRUE;
END; (* IsPaint *)

PROCEDURE GetGeosImage;
(* Read chain into geos_image *)
 VAR ch: CHAR; cnt, i: BYTE; res: WORD;
  cbuf: ARRAY [0..7] OF BYTE;
BEGIN
 err_fp := -1;
 err_ip := -1;
 err_str := '';
 WITH geos_image^ DO BEGIN
  sz := 0;
  FillChar(pat, SizeOf(pat), 0);
  FillChar(gap, SizeOf(gap), 0);
  FillChar(col, SizeOf(col), BW_COLOR);
  IF NOT cvt_chain_used(chain) THEN
   BEGIN (* empty chain *)
    sz := FrmSize;
    EXIT;
   END; (* if *)
  cvt_chain(chain);
  WHILE cvt_getch(ch) DO BEGIN
   cnt := ORD(ch);
   CASE cnt OF
    $00..$40:
     BEGIN (* individual bytes *)
      IF (cnt = 0) AND NOT cvt_eoc THEN
       IF err_str = '' THEN BEGIN
        err_fp := cvt_pos - 1;
        err_ip := sz;
        err_str := 'ind[0] at'+long2str(err_fp)+' in #'+long2str(chain);
       END; (* if *)
      cvt_read(raw[sz], cnt, res);
      INC(sz, res);
      IF res <> cnt THEN BEGIN
       IF err_str = '' THEN
        err_str := '#'+long2str(chain)+' too short';
       EXIT;
      END; (* if *)
     END;
    $41..$7F:
     BEGIN (* fill patterns *)
      cnt := cnt AND $3F;
      cvt_read(cbuf, SizeOf(cbuf), res);
      IF res <> SizeOf(cbuf) THEN BEGIN
       IF err_str = '' THEN
        err_str := '#'+long2str(chain)+' too short';
       EXIT;
      END; (* if *)
      FOR i := 1 TO cnt DO BEGIN
       Move(cbuf, raw[sz], SizeOf(cbuf));
       Inc(sz, SizeOf(cbuf));
      END; (* for *)
     END;
    $80..$FF:
     BEGIN (* repeat *)
      cnt := cnt AND $7F;
      IF (cnt = 0) AND (err_str <> '') THEN BEGIN
       err_fp := cvt_pos - 1;
       err_ip := sz;
       err_str := 'rep[0] at '+long2str(err_fp)+' in #'+long2str(chain);
      END; (* if *)
      IF NOT cvt_getch(ch) THEN BEGIN
       IF err_str = '' THEN
        err_str := '#'+long2str(chain)+' too short';
       EXIT;
      END; (* if *)
      FillChar(raw[sz], cnt, ch);
      Inc(sz, cnt);
     END;
   END; (* case *)
   IF sz > ErrSize THEN BEGIN
    IF err_str = '' THEN
     err_str := '#'+long2str(chain)+' too long';
    EXIT;
   END; (* if *)
  END; (* while *)
  IF err_str = '' THEN
   IF sz = PatSize THEN
    (* black/white *)
    sz := FrmSize
   ELSE IF sz = FrmSize THEN
    BEGIN (* color *)
     FOR i := 0 TO 7 DO
      IF gap[i] <> 0 THEN BEGIN
       err_str := 'gap not null in #'+long2str(chain);
       EXIT;
      END (* if *)
    END
   ELSE
    err_str := '#'+long2str(chain)+' has wrong size';
 END; (* with *)
END; (* GetGeosImage *)

PROCEDURE GeosImage2Image;
(* convert geos_image to BitMap *)
 VAR r, c, n, i, ip: INTEGER;
  col, a, b: BYTE;
BEGIN
 IF geos_image^.sz = FrmSize THEN
  BEGIN
   BitMap^.x := 639;
   BitMap^.y :=  15;
   FOR r := 0 TO 1 DO
    FOR c := 0 TO 79 DO BEGIN
     col := geos_image^.col[r,c];
     FOR i := 0 TO 3 DO BEGIN
      CASE col AND $88 OF
       $00: BEGIN a := $00; b := $00; END;
       $08: BEGIN a := $FF; b := $FF; END;
       $80: BEGIN a := $FF; b := $00; END;
       $88: BEGIN a := $00; b := $FF; END;
      END; (* case *)
      FOR n := 0 TO 7 DO
       BitMap^.p[8*r+n,i,c] := (geos_image^.pat[r,c,n] AND a) XOR b;
      col := (col AND $77) SHL 1;
     END; (* for *)
    END; (* for *)
  END
 ELSE (* geos_image^.sz <> FrmSize *)
  BEGIN
   IF err_ip < 0 THEN BEGIN
    err_ip := geos_image^.sz;
    IF err_ip > 1280 THEN err_ip := 1280;
   END; (* if *)
   BitMap^.x := 639;
   BitMap^.y :=  31;
   ip := 0;
   FOR r := 0 TO 3 DO
    FOR c := 0 TO 79 DO
     FOR n := 0 TO 7 DO BEGIN
      IF ip < err_ip THEN
       col := $10
      ELSE IF ip < geos_image^.sz THEN
       col := $A6
      ELSE
       col := $00;
      INC(ip);
      FOR i := 0 TO 3 DO BEGIN
       CASE col AND $88 OF
        $00: BitMap^.p[8*r+n,i,c] := $00;
        $08: BitMap^.p[8*r+n,i,c] := NOT geos_image^.err[r,c,n];
        $80: BitMap^.p[8*r+n,i,c] := geos_image^.err[r,c,n];
        $88: BitMap^.p[8*r+n,i,c] := $FF;
       END; (* case *)
       col := (col AND $77) SHL 1;
      END; (* for *)
     END; (* for *)
  END; (* else *)
END; (* GeosImage2Image *)

FUNCTION ReadChar: CHAR;
 VAR ch, dummy: CHAR;
BEGIN
 REPEAT
  ch := ReadKey;
  IF ch = #0 THEN dummy := ReadKey;
 UNTIL ch <> #0;
 ReadChar := ch;
END; (* ReadChar *)

PROCEDURE CorrectByte(fp: LongInt; b: BYTE);
BEGIN
 Buffer^[fp] := CHR(b);
 GetGeosImage;
 GeosImage2Image;
 PutImage(0, y, BitMap^, NormalPut);
 IF BitMap^.y = 15 THEN BEGIN
  SetFillStyle(SolidFill, 0);
  Bar(0, y+16, VGA_MAXX, y+31);
 END; (* if *)
END; (* CorrectByte *)

FUNCTION CorrectRepeater(fp: LongInt): BOOLEAN;
 VAR len: LongInt; ch: CHAR;
BEGIN
 Buffer^[fp] := #$80;
 GetGeosImage;
 WITH geos_image^ DO
  IF sz < PatSize THEN
   len := PatSize - sz
  ELSE IF sz < FrmSize THEN
   len := FrmSize - sz
  ELSE (* sz > FrmSize *)
   len := 1;
 CorrectRepeater := FALSE;
 REPEAT
  IF len < 1   THEN len := 1;
  IF len > $7F THEN len := $7F;
  CorrectByte(fp, $80 OR len);
  TextOut('REP: <+> +1  <-> -1  <*> +8  </> -8  <y>es  <n>o  len='
          +long2str(len)+'  size='+long2str(geos_image^.sz));
  REPEAT
   ch := UpCase(ReadChar);
  UNTIL ch IN ['+', '-', '*', '/', 'Y', 'N'];
  CASE ch OF
   'Y': CorrectRepeater := TRUE;
   '+': INC(len);
   '*': INC(len, 8);
   '-': DEC(len);
   '/': DEC(len, 8);
   'N': CorrectByte(fp,0);
  END; (* case *)
 UNTIL (ch = 'Y') OR (ch = 'N');
END; (* CorrectRepeater *)

FUNCTION CorrectPattern(fp: LongInt): BOOLEAN;
 VAR cnt: LongInt; ch: CHAR;
BEGIN
 Buffer^[fp] := #$41;
 GetGeosImage;
 WITH geos_image^ DO
  IF sz < PatSize THEN
   cnt := PatSize - sz
  ELSE IF sz < FrmSize THEN
   cnt := FrmSize - sz
  ELSE (* sz > FrmSize *)
   cnt := 8;
 cnt := 1 + ((cnt+7) DIV 8);
 CorrectPattern := FALSE;
 REPEAT
  IF cnt < 1   THEN cnt := 1;
  IF cnt > $3F THEN cnt := $3F;
  CorrectByte(fp, $40 OR cnt);
  TextOut('PAT: <+> +1  <-> -1  <*> +8  </> -8  <y>es  <n>o  cnt='
          +long2str(cnt)+'  size='+long2str(geos_image^.sz));
  REPEAT
   ch := UpCase(ReadChar);
  UNTIL ch IN ['+', '-', '*', '/', 'Y', 'N'];
  CASE ch OF
   'Y': CorrectPattern := TRUE;
   '+': INC(cnt);
   '*': INC(cnt, 8);
   '-': DEC(cnt);
   '/': DEC(cnt, 8);
   'N': CorrectByte(fp,0);
  END; (* case *)
 UNTIL (ch = 'Y') OR (ch = 'N');
END; (* CorrectPattern *)

FUNCTION CorrectIndividual(fp: LongInt): BOOLEAN;
 VAR cnt: LongInt; ch: CHAR;
BEGIN
 cnt := 0;
 WITH geos_image^ DO
  REPEAT
   INC(cnt);
   TextOut('IND: '+long2str(cnt));
   Buffer^[fp] := CHR(cnt);
   GetGeosImage;
  UNTIL (cnt > $40) OR (sz = PatSize) OR (sz = FrmSize);
 IF cnt > $40 THEN BEGIN
  cnt := 1;
  Buffer^[fp] := CHR(cnt);
  GetGeosImage;
 END; (* if *)
 CorrectIndividual := FALSE;
 REPEAT
  IF cnt < 1   THEN cnt := 1;
  IF cnt > $40 THEN cnt := $40;
  CorrectByte(fp, cnt);
  TextOut('IND: <+> +1  <-> -1  <*> +5  </> -5  <s>earch  <y>es  <n>o'
          +'  cnt='+long2str(cnt)+'  size='+long2str(geos_image^.sz));
  REPEAT
   ch := UpCase(ReadChar);
  UNTIL ch IN ['+', '-', '*', '/', 'S', 'Y', 'N'];
  CASE ch OF
   'Y': CorrectIndividual := TRUE;
   '+': INC(cnt);
   '*': INC(cnt, 5);
   '-': DEC(cnt);
   '/': DEC(cnt, 5);
   'S': BEGIN
     WITH geos_image^ DO
      REPEAT
       INC(cnt);
       TextOut('IND: '+long2str(cnt));
       Buffer^[fp] := CHR(cnt);
       GetGeosImage;
      UNTIL (cnt > $40) OR (sz = PatSize) OR (sz = FrmSize);
    END;
   'N': CorrectByte(fp,0);
  END; (* case *)
 UNTIL (ch = 'Y') OR (ch = 'N');
END; (* CorrectIndividual *)

PROCEDURE DoCorrection(VAR level: WORD);
 VAR ch: CHAR; corr_fp: LongInt;
BEGIN
 IF NOT do_view THEN BEGIN
  level := 3;
  EXIT;
 END; (* if *)
 IF err_str <> ''
  THEN TextOut('CORRECT: '+err_str+'  [Press ''?'' for menue]')
  ELSE TextOut('CORRECT: no error  [Press ''?'' for menue]');
 REPEAT
  ch := UpCase(ReadChar);
  CASE ch OF
   '?':
    TextOut('<c>orrect  <g>ap  <l>evel  <e>rror  <o>k');
   'E':
    IF err_str <> ''
     THEN TextOut('CORRECT: '+err_str+'  [Press ''?'' for menue]')
     ELSE TextOut('CORRECT: no error  [Press ''?'' for menue]');
   'L': BEGIN
      TextOut('Level: <1> stop  <2> continue  <3> continue b/w  <b>ack');
      REPEAT
       ch := UpCase(ReadChar);
      UNTIL ch in ['1'..'3','B'];
      IF ch <> 'B' THEN BEGIN
       level := ORD(ch) - ORD('0');
       TextOut(cvt_name);
       EXIT;
      END; (* if *)
      TextOut('<c>orrect  <g>ap  <l>evel  <e>rror  <o>k');
     END;
   'G':
    WITH geos_image^ DO
     IF sz = FrmSize THEN BEGIN
      FillChar(gap, SizeOf(gap), 0);
      level := 0;
      GeosImage2Image;
      ch := 'O';
     END; (* if *)
   'C':
    IF err_fp < 0 THEN
     TextOut('ERROR: no error file position found  [Press ''?'' for menue]')
    ELSE BEGIN
     corr_fp := err_fp;
     REPEAT
      TextOut('<r>epeater  <p>attern  <i>ndividual  <b>ack  size='+long2str(geos_image^.sz));
      ch := UpCase(ReadKey);
      CASE ch OF
       '?': TextOut('<r>epeater  <f>iller  <i>ndividual  <b>ack');
       'R': IF CorrectRepeater(corr_fp) THEN ch := 'O';
       'P': IF CorrectPattern(corr_fp) THEN ch := 'O';
       'I': IF CorrectIndividual(corr_fp) THEN ch := 'O';
       'O': ch := #0;
      END; (* case *)
     UNTIL (ch = 'B') OR (ch = 'O');
     TextOut('<c>orrect  <g>ap  <l>evel  <e>rror  <o>k');
    END; (* else *)
  END; (* case *)
 UNTIL ch = 'O';
 TextOut(cvt_name);
END; (* DoCorrection *)

PROCEDURE OutputGeosImage;
 VAR level: WORD;
BEGIN
 REPEAT
  GetGeosImage;
  level := err_level * ORD(err_str <> '');
  GeosImage2Image;
  IF do_view THEN
   PutImage(0, y, BitMap^, NormalPut);
  IF level = 4 THEN
   DoCorrection(level);
 UNTIL level <> 4;
 IF (level = 2) THEN BEGIN
  IF do_view THEN TextOut('*** '+err_str);
  BitMap^.y := 15;
 END; (* if *)
 IF level = 3 THEN BEGIN
  IF do_view THEN TextOut('*** '+err_str);
  geos_image^.sz := FrmSize;
  GeosImage2Image;
 END; (* if *)
 IF do_pcx THEN
  pcx_image(BitMap^);
 IF level = 1 THEN BEGIN
  IF do_pcx THEN pcx_close;
   G_FATAL(err_str);
 END; (* if *)
END; (* OutputGeosImage *)

PROCEDURE DoPaint(pcx_name: STRING; view: BOOLEAN; lvl: WORD);
 VAR dy: LongInt; max_chain: ShortInt;
BEGIN
 do_view := view;
 do_pcx := pcx_name <> '';
 IF lvl > 4 THEN lvl := 4;
 err_level := lvl;
 y := -16;
 New(geos_image);
 New(BitMap);
 (* get max_chain *)
 max_chain := 45;
 REPEAT
  DEC(max_chain);
  IF max_chain < 0 THEN
   FATAL('empty painting');
 UNTIL cvt_chain_used(max_chain);
 (* open *)
 IF do_view THEN
  BEGIN
   OpenGraphic(CBMcp);
   TextOut(cvt_name);
  END
 ELSE
  Write(cvt_name,' ');
 IF do_pcx THEN
  pcx_open(pcx_name, CBMcp, VGA_MAXX+1);
 (* output loop *)
 chain := 0;
 WHILE chain <= max_chain DO BEGIN
  IF do_view THEN
   BEGIN
    INC(y, 16);
    IF y > VGA_MAXY-50 THEN BEGIN
     dy := y DIV 2;
     ScrollUp(0, 0, VGA_MAXX, y, dy);
     DEC(y, dy);
    END; (* if *)
   END
  ELSE
   Write('.');
  OutputGeosImage;
  Inc(chain);
 END; (* while *)
 (* close *)
 IF do_pcx THEN
  pcx_close;
 IF do_view
  THEN CloseGraphic(TRUE)
  ELSE WriteLn;
 Dispose(geos_image);
 Dispose(BitMap);
END; (* DoPaint *)

END. (* Paint *)