(* PHOTO.PAS -- translate photo scrap/album
** Copyright (C) 1995,1996  Jochen Metzinger
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

UNIT Photo;

INTERFACE

USES Crt, Graph,
 graphic, global, cvt, pcx;

CONST
 BW_COLOR = $BF;
  (* dark gray on lt grey *)

FUNCTION IsPhotoScrap(VAR info: InfoRec): BOOLEAN;
(* is it a Photo Scrap *)
FUNCTION IsPhotoAlbum(VAR info: InfoRec): BOOLEAN;
(* is it a Photo Scrap *)
FUNCTION IsWrite(VAR info: InfoRec): BOOLEAN;
(* is it a geoWrite image *)

PROCEDURE OpenScrap(view: BOOLEAN; lvl: WORD);
(* init translation *)
PROCEDURE DoScrap(_chain: ShortInt; pcx_name: STRING);
(* do translation for a chain *)
PROCEDURE CloseScrap;
(* close translation *)

IMPLEMENTATION

CONST
 MAX_SCAP = 40*(200 + 200 DIV 8);

TYPE
 ScrapType = RECORD
  w: BYTE; h: WORD;
  sz: WORD;
  data: ARRAY [0..MAX_SCAP+128] OF BYTE;
  good: BOOLEAN; wh: WORD; ssz: WORD;
 END;
 ImageType = RECORD
  x, y: INTEGER;
  p: ARRAY [0..8*4*40] OF BYTE;
  __res: WORD;
 END; (* ImageType *)

VAR
 chain: ShortInt;
  (* choosen chain *)
 ScrapImage: ^ScrapType;
  (* scrap image / GEOS *)
 BitMap: ^ImageType;
  (* scrap part (8x..) / EGA-Image *)
 err_fp, err_ip: LongInt; err_str: STRING;
  (* Errors from GetScrapImage *)
 do_view: BOOLEAN;
  (* graphical output? *)
 err_level: WORD;
  (* level of error recovery *)

PROCEDURE GetScrapImage;
 VAR ch: CHAR; cnt: BYTE; res: WORD;
  b_beg: LongInt; b_rep, b_pos, b_sz: BYTE;
 FUNCTION next(VAR ch: CHAR): BOOLEAN;
 (* getchar() with `buffer' *)
 BEGIN
  next := cvt_getch(ch);
  IF b_rep <> 0 THEN BEGIN
   INC(b_pos);
   IF b_pos = b_sz THEN BEGIN
    DEC(b_rep);
    b_pos := 0;
    cvt_seek(b_beg);
   END; (* if *)
  END; (* if *)
 END; (* next *)
BEGIN (* GetScrapImage *)
 err_fp := -1;
 err_ip := -1;
 err_str := '';
 WITH ScrapImage^ DO BEGIN
  w := 0; h := 0; sz := 0;
  FillChar(data, SizeOf(data), 0);
  good := FALSE;
  cvt_chain(chain);
  IF NOT cvt_getch(ch) THEN BEGIN
   err_str := '$'+HexStr(chain,2)+' too short';
   EXIT;
  END; (* if *)
  w := ORD(ch);
  IF w = 0 THEN BEGIN
   err_str := 'scrap $'+HexStr(chain,2)+' has no width';
   EXIT;
  END; (* if *)
  IF w > 40 THEN BEGIN
   err_str := 'scrap $'+HexStr(chain,2)+' too wide';
   EXIT;
  END; (* if *)
  cvt_read(h, SizeOf(h), res);
  IF res <> SizeOf(h) THEN BEGIN
   err_str := 'chain $'+HexStr(chain,2)+' too short';
   EXIT;
  END; (* if *)
  IF h > 200 THEN BEGIN
   err_str := 'scrap $'+HexStr(chain,2)+' too high';
   EXIT;
  END; (* if *)
  wh := w*h;
  ssz := wh + w*((h+7) DIV 8);
  b_beg:= -1;
  b_rep := 0;
  b_pos := 0;
  b_sz := 0;
  WHILE next(ch) DO BEGIN
   cnt := ORD(ch);
   CASE cnt OF
    $00..$7F:
     BEGIN (* repeat *)
      IF cnt = 0 THEN BEGIN
       err_fp := cvt_pos - 1;
       err_ip := sz;
       err_str := 'rep[0] at '+long2str(err_fp)+' in $'+HexStr(chain,2);
      END; (* if *)
      IF NOT next(ch) THEN BEGIN
       IF err_str = '' THEN
        err_str := 'chain $'+HexStr(chain,2)+' too short';
       EXIT;
      END; (* if *)
      FillChar(data[sz], cnt, ch);
      Inc(sz, cnt);
     END;
    $80..$DC:
     BEGIN (* individual bytes *)
      DEC(cnt, 128);
      IF cnt = 0 THEN
       IF err_str = '' THEN BEGIN
        err_fp := cvt_pos - 1;
        err_ip := sz;
        err_str := 'ind[0] at '+long2str(err_fp)+' in $'+HexStr(chain,2);
       END; (* if *)
      WHILE cnt > 0 DO BEGIN
       IF NOT next(CHAR(data[sz])) THEN BEGIN
        IF err_str = '' THEN
         err_str := 'chain $'+HexStr(chain,2)+' too short';
        EXIT;
       END; (* if *)
       INC(sz); DEC(cnt);
      END; (* while *)
     END;
    $DD..$FF:
     BEGIN (* compress *)
      IF b_rep <> 0 THEN BEGIN
       IF err_str = '' THEN
        err_str := 'com[] within compression in $'+HexStr(chain,2);
        EXIT;
       END; (* if *)
      b_sz := cnt - 219;
      IF NOT cvt_getch(ch) THEN BEGIN
       err_str := 'chain $'+HexStr(chain,2)+' too short';
       EXIT;
      END; (* if *)
      b_rep := ORD(ch);
      IF b_rep = 0 THEN
       IF err_str = '' THEN BEGIN
        err_fp := cvt_pos - 1;
        err_ip := sz;
        err_str := 'com[0] at '+long2str(err_fp)+' in $'+HexStr(chain,2);
       END; (* if *)
      b_beg := cvt_pos;
      b_pos := 0;
     END;
   END; (* case *)
   IF sz > 40*200 THEN BEGIN
    IF err_str = '' THEN
     err_str := 'chain $'+HexStr(chain,2)+' too long';
    EXIT;
   END; (* if *)
  END; (* while *)
  IF err_str = '' THEN
   IF sz = ssz THEN
    good := TRUE
   ELSE
    err_str := 'chain $'+HexStr(chain,2)+' has wrong size';
 END; (* with *)
END; (* GetScrapImage *)

PROCEDURE ScrapImage2Image(r: WORD);
(* convert ScrapImage to BitMap *)
 VAR ip, ipp, cp, idx: WORD;
  n, c, i: BYTE; col, a, b: BYTE;
BEGIN
 WITH ScrapImage^, BitMap^ DO BEGIN
  x := (w SHL 3)-1;
  y := 7;
  FillChar(p, SizeOf(p), 0);
  ip := (w*r) SHL 3;
  IF good THEN
   BEGIN
    cp := wh + (ip SHR 3);
    FOR c := 0 TO w-1 DO BEGIN
     col := data[cp]; INC(cp);
     FOR i := 0 TO 3 DO BEGIN
      CASE col AND $88 OF
       $00: BEGIN a := $00; b := $00; END;
       $08: BEGIN a := $FF; b := $FF; END;
       $80: BEGIN a := $FF; b := $00; END;
       $88: BEGIN a := $00; b := $FF; END;
      END; (* case *)
      idx := w*i + c; ipp := ip;
      FOR n := 0 TO 7 DO BEGIN
       p[idx] := (data[ipp] AND a) XOR b;
       INC(idx, w SHL 2); INC(ipp, w);
      END; (* for *)
      col := (col AND $77) SHL 1;
     END; (* for *)
     INC(ip);
    END; (* for *)
    IF r = h SHR 3 THEN
     y := h AND $0F;
   END
  ELSE (* NOT good *)
   BEGIN
    FOR n := 0 TO 7 DO
     FOR c := 0 TO w-1 DO BEGIN
      IF ip < err_ip THEN
       col := $10
      ELSE IF ip < sz THEN
       col := $A6
      ELSE
       col := $00;
      idx := 4*n*w+c;
      FOR i := 0 TO 3 DO BEGIN
       CASE col AND $88 OF
        $00: p[idx] := $00;
        $08: p[idx] := NOT data[ip];
        $80: p[idx] := data[ip];
        $88: p[idx] := $FF;
       END; (* case *)
       col := (col AND $77) SHL 1;
       INC(idx, w);
      END; (* for *)
      INC(ip);
     END; (* for *)
   END; (* else *)
 END; (* with *)
END; (* ScrapImage2Image *)

PROCEDURE ViewScrap;
 VAR i, max: WORD;
BEGIN
 ClearDevice;
 WITH ScrapImage^ DO BEGIN
  IF err_ip < 0 THEN
   err_ip := sz;
  IF err_ip > ssz THEN
   err_ip := ssz;
  max := sz DIV w;
  IF max > VGA_MAXY-10 THEN
   max := VGA_MAXY-10;
  FOR i := 0 TO max SHR 3 DO BEGIN
   ScrapImage2Image(i);
   PutImage(0, i SHL 3, BitMap^, NormalPut);
  END; (* for *)
 END; (* with *)
END; (* ViewScrap *)

FUNCTION ReadChar: CHAR;
 VAR ch, dummy: CHAR;
BEGIN
 REPEAT
  ch := ReadKey;
  IF ch = #0 THEN dummy := ReadKey;
 UNTIL ch <> #0;
 ReadChar := ch;
END; (* ReadChar *)

PROCEDURE CorrectByte(fp: LongInt; b: BYTE);
 VAR i: INTEGER;
BEGIN
 Buffer^[fp] := CHR(b);
 GetScrapImage;
 ViewScrap;
END; (* CorrectByte *)

FUNCTION CorrectRepeater(fp: LongInt): BOOLEAN;
 VAR len: LongInt; ch: CHAR;
BEGIN
 WITH ScrapImage^ DO
  IF sz < ssz THEN
   len := ssz - sz
  ELSE (* sz > ssz *)
   len := 1;
 CorrectRepeater := FALSE;
 REPEAT
  IF len < 1   THEN len := 1;
  IF len > $7F THEN len := $7F;
  CorrectByte(fp, len);
  TextOut('REP: <+> +1  <-> -1  <*> +8  </> -8  <y>es  <n>o  len='
          +long2str(len)+'  size='+long2str(ScrapImage^.sz)+'/'
          +long2str(ScrapImage^.ssz));
  REPEAT
   ch := UpCase(ReadChar);
  UNTIL ch IN ['+', '-', '*', '/', 'Y', 'N'];
  CASE ch OF
   'Y': CorrectRepeater := TRUE;
   '+': INC(len);
   '*': INC(len, 8);
   '-': DEC(len);
   '/': DEC(len, 8);
   'N': CorrectByte(fp,0);
  END; (* case *)
 UNTIL (ch = 'Y') OR (ch = 'N');
END; (* CorrectRepeater *)

FUNCTION CorrectIndividual(fp: LongInt): BOOLEAN;
 VAR cnt: LongInt; ch: CHAR;
BEGIN
 cnt := $80;
 WITH ScrapImage^ DO
  REPEAT
   INC(cnt);
   TextOut('IND: '+long2str(cnt AND $7F));
   Buffer^[fp] := CHR(cnt);
   GetScrapImage;
  UNTIL good OR (cnt > $DC);
 IF NOT ScrapImage^.good THEN
  cnt := $81;
 CorrectIndividual := FALSE;
 REPEAT
  IF cnt < $81 THEN cnt := $81;
  IF cnt > $DC THEN cnt := $DC;
   CorrectByte(fp, cnt);
   TextOut('IND: <+>1 <->1 <*>+5 </>-5  <s>earch  <y>es <n>o'
          +' cnt='+long2str(cnt AND $7F)+' size='+long2str(ScrapImage^.sz)
          +'/'+long2str(ScrapImage^.ssz));
  REPEAT
   ch := UpCase(ReadChar);
  UNTIL ch IN ['+', '-', '*', '/', 'S', 'Y', 'N'];
  CASE ch OF
   'Y': CorrectIndividual := TRUE;
   '+': INC(cnt);
   '*': INC(cnt, 5);
   '-': DEC(cnt);
   '/': DEC(cnt, 5);
   'S': BEGIN
         WITH ScrapImage^ DO
          REPEAT
           INC(cnt);
           TextOut('IND: '+long2str(cnt AND $7F));
           Buffer^[fp] := CHR(cnt);
           GetScrapImage;
          UNTIL (cnt > $DC) OR (sz = ssz);
         IF cnt > $DC THEN cnt := $DC;
        END;
   'N': CorrectByte(fp,0);
  END; (* case *)
 UNTIL (ch = 'Y') OR (ch = 'N');
END; (* CorrectIndividual *)

FUNCTION CorrectPattern(fp: LongInt): BOOLEAN;
 VAR cnt: LongInt; ch: CHAR;
BEGIN
 cnt := $DC;
 WITH ScrapImage^ DO
  REPEAT
   INC(cnt);
   TextOut('PAT: '+long2str(cnt-$DC));
   Buffer^[fp] := CHR(cnt);
   GetScrapImage;
  UNTIL good OR (cnt > $FF);
 IF NOT ScrapImage^.good THEN
  cnt := $DD;
 CorrectPattern := FALSE;
 REPEAT
  IF cnt < $DD THEN cnt := $DD;
  IF cnt > $FF THEN cnt := $FF;
  CorrectByte(fp, cnt);
  TextOut('PAT: <+>1 <->1 <*>+5 </>-5  <s>earch  <y>es <n>o'
         +' cnt='+long2str(cnt-$DC)+' size='+long2str(ScrapImage^.sz)
         +'/'+long2str(ScrapImage^.ssz));
  REPEAT
   ch := UpCase(ReadChar);
  UNTIL ch IN ['+', '-', '*', '/', 'S', 'Y', 'N'];
  CASE ch OF
   'Y': CorrectPattern := TRUE;
   '+': INC(cnt);
   '*': INC(cnt, 5);
   '-': DEC(cnt);
   '/': DEC(cnt, 5);
   'S': BEGIN
         WITH ScrapImage^ DO
          REPEAT
           INC(cnt);
           TextOut('PAT: '+long2str(cnt-$DC));
           Buffer^[fp] := CHR(cnt);
           GetScrapImage;
          UNTIL good OR (cnt > $FF);
         IF NOT ScrapImage^.good THEN cnt := $FF;
        END;
   'N': CorrectByte(fp,0);
  END; (* case *)
 UNTIL (ch = 'Y') OR (ch = 'N');
END; (* CorrectPattern *)

FUNCTION CorrectSearch(fp: LongInt): BOOLEAN;
 VAR cnt: LongInt; ch: CHAR;
BEGIN
 cnt := $00;
 WITH ScrapImage^ DO
  REPEAT
   INC(cnt);
   TextOut('SRCH: '+long2str(cnt));
   Buffer^[fp] := CHR(cnt);
   GetScrapImage;
  UNTIL good OR (cnt > $FF);
 IF NOT ScrapImage^.good THEN
  cnt := $00;
 CorrectSearch := FALSE;
 REPEAT
  IF cnt < $00 THEN cnt := $00;
  IF cnt > $FF THEN cnt := $FF;
  CorrectByte(fp, cnt);
  TextOut('SRCH: <+>1 <->1 <*>+8 </>-8  <s>earch  <y>es <n>o'
          +' cnt='+long2str(cnt)+' size='+long2str(ScrapImage^.sz)
          +'/'+long2str(ScrapImage^.ssz));
  REPEAT
   ch := UpCase(ReadChar);
  UNTIL ch IN ['+', '-', '*', '/', 'S', 'Y', 'N'];
  CASE ch OF
   'Y': CorrectSearch := TRUE;
   '+': INC(cnt);
   '*': INC(cnt, 8);
   '-': DEC(cnt);
   '/': DEC(cnt, 8);
   'S': BEGIN
         WITH ScrapImage^ DO
          REPEAT
           INC(cnt);
           TextOut('SRCH: '+long2str(cnt));
           Buffer^[fp] := CHR(cnt);
           GetScrapImage;
          UNTIL good OR (cnt > $FF);
         IF NOT ScrapImage^.good THEN cnt := $FF;
        END;
   'N': CorrectByte(fp,0);
  END; (* case *)
 UNTIL (ch = 'Y') OR (ch = 'N');
END; (* CorrectSearch *)

PROCEDURE DoCorrection(VAR level: WORD);
 VAR ch: CHAR; corr_fp: LongInt;
BEGIN
 IF NOT do_view THEN BEGIN
  level := 3;
  EXIT;
 END; (* if *)
 IF err_str <> ''
  THEN TextOut('CORRECT: '+err_str+'  [Press ''?'' for menue]')
  ELSE TextOut('CORRECT: no error  [Press ''?'' for menue]');
 REPEAT
  ch := UpCase(ReadChar);
  CASE ch OF
   '?':
    TextOut('<c>orrect  <l>evel  <e>rror  <o>k');
   'E':
    IF err_str <> ''
     THEN TextOut('CORRECT: '+err_str+'  [Press ''?'' for menue]')
     ELSE TextOut('CORRECT: no error  [Press ''?'' for menue]');
   'L': BEGIN
      TextOut('Level: <1> stop  <2> continue  <3> continue b/w  <b>ack');
      REPEAT
       ch := UpCase(ReadChar);
      UNTIL ch in ['1'..'3','B'];
      IF ch <> 'B' THEN BEGIN
       level := ORD(ch) - ORD('0');
       TextOut(cvt_name);
       EXIT;
      END; (* if *)
      TextOut('<c>orrect  <l>evel  <e>rror  <o>k');
     END;
   'C':
    IF err_fp < 0 THEN
     TextOut('CORRECT: no error file position found  [Press ''?'' for menue]')
    ELSE BEGIN
     corr_fp := err_fp;
     REPEAT
      TextOut('<r>epeater  <i>ndividual  <p>attern  <s>earch  <b>ack  size='
            +long2str(ScrapImage^.sz)+'/'+long2str(ScrapImage^.ssz));
      ch := UpCase(ReadKey);
      CASE ch OF
       '?': TextOut('<r>epeater  <f>iller  <i>ndividual  <b>ack');
       'R': IF CorrectRepeater(corr_fp) THEN ch := 'O';
       'P': IF CorrectPattern(corr_fp) THEN ch := 'O';
       'I': IF CorrectIndividual(corr_fp) THEN ch := 'O';
       'S': IF CorrectSearch(corr_fp) THEN ch := 'O';
       'O': ch := #0;
      END; (* case *)
     UNTIL (ch = 'B') OR (ch = 'O');
     TextOut('<c>orrect  <l>evel  <e>rror  <o>k');
    END; (* else *)
  END; (* case *)
 UNTIL ch = 'O';
 TextOut(cvt_name+' $'+HexStr(chain,2));
END; (* DoCorrection *)

PROCEDURE DoScrap(_chain: ShortInt; pcx_name: STRING);
 VAR level, i: WORD;
BEGIN
 chain := _chain;
 REPEAT
  GetScrapImage;
  IF err_str = ''
   THEN level := 0
   ELSE level := err_level;
  IF do_view THEN
   ViewScrap;
  IF level = 4 THEN
   DoCorrection(level);
 UNTIL level <> 4;
 IF (level = 2) AND do_view THEN
   TextOut('*** '+err_str);
 IF level = 3 THEN
  WITH ScrapImage^ DO BEGIN
   sz := ssz; good := TRUE;
   FOR i := wh TO ssz-1 DO
    data[i] := BW_COLOR;
   IF do_view THEN BEGIN
    ViewScrap;
    TextOut('*** '+err_str);
   END; (* if *)
  END; (* with *)
 IF pcx_name <> '' THEN BEGIN
  pcx_open(pcx_name, CBMcp, ScrapImage^.w SHL 3);
  WITH ScrapImage^ DO
   FOR i := 0 TO (sz DIV w) SHR 3 DO BEGIN
    ScrapImage2Image(i);
    pcx_image(BitMap^);
   END; (* for *)
  pcx_close;
 END; (* if *)
 IF level = 1 THEN
  G_FATAL(err_str);
 IF do_view THEN BEGIN
  IF level = 0 THEN
    TextOut(cvt_name+' $'+HexStr(chain,2)+' DONE');
  REPEAT UNTIL ReadKey <> #0;
 END; (* if *)
END; (* DoScrap *)

PROCEDURE OpenScrap(view: BOOLEAN; lvl: WORD);
BEGIN
 do_view := view;
 IF lvl = 0 THEN lvl := 1;
 IF lvl > 4 THEN lvl := 4;
 err_level := lvl;
 IF do_view THEN BEGIN
   OpenGraphic(CBMcp);
   TextOut(cvt_name);
 END; (* if *)
 New(ScrapImage);
 New(BitMap);
END; (* OpenScrap *)

PROCEDURE CloseScrap;
BEGIN
 Dispose(BitMap);
 Dispose(ScrapImage);
 IF do_view
  THEN CloseGraphic(FALSE);
END; (* CloseScrap *)

FUNCTION IsPhotoScrap(VAR info: InfoRec): BOOLEAN;
BEGIN
 IsPhotoScrap := FALSE;
 WITH info DO BEGIN
  (* USR *)
  IF ftyp AND $07 <> $03 THEN EXIT;
  (* SYS *)
  IF gtyp <> $04 THEN EXIT;
  (* SEQ *)
  IF strk <> 0 THEN EXIT;
  (* Class: 'Photo Scrap V1.1' *)
  IF GetString(class,15) <> 'Photo Scrap V1.' THEN EXIT;
 END; (* with *)
 IsPhotoScrap := TRUE;
END; (* IsPhotoScrap *)

FUNCTION IsPhotoAlbum(VAR info: InfoRec): BOOLEAN;
BEGIN
 IsPhotoAlbum := FALSE;
 WITH info DO BEGIN
  (* USR *)
  IF ftyp AND $07 <> $03 THEN EXIT;
  (* DOC *)
  IF gtyp <> $07 THEN EXIT;
  (* VLIR *)
  IF strk <> 1 THEN EXIT;
  (* Class: 'Photo Album V1.0' *)
  IF GetString(class,15) <> 'photo album V1.' THEN EXIT;
 END; (* with *)
 IsPhotoAlbum := TRUE;
END; (* IsPhotoScrap *)

FUNCTION IsWrite(VAR info: InfoRec): BOOLEAN;
BEGIN
 IsWrite := FALSE;
 WITH info DO BEGIN
  (* USR *)
  IF ftyp AND $07 <> $03 THEN EXIT;
  (* DOC *)
  IF gtyp <> $07 THEN EXIT;
  (* VLIR *)
  IF strk <> 1 THEN EXIT;
  (* Class: 'Write Image 1.1' *)
  IF GetString(class,15) <> 'Write Image V1.' THEN EXIT;
 END; (* with *)
 IsWrite := TRUE;
END; (* GetStyle *)

END. (* Photo *)