(* SHOWFONT.PAS -- show GEOS fonts
** Copyright (C) 1995,1996  Jochen Metzinger
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

PROGRAM ShowFont;

USES Graph, global, fonts, graphic, cvt;

CONST
 NameColor = LightGray;
 BoxColor  = Red;
 CharColor = White;

TYPE
 FontHeader =
  RECORD
    ul: BYTE; (* Pixels minus 1 above the underline. *)
   bss: WORD; (* Bytes in the bit stream *)
   pnt: BYTE; (* Point size, character height in pixels *)
   idx: WORD; (* Index to table of bit stream indices *)
    bs: WORD; (* Index to first bit stream. *)
  END;

 FontIndices =
  ARRAY [32..127] OF WORD;
  (* Indices for char ' ' to '~' *)

 FontBitStream =
  ARRAY [0..BufferMaxSize] OF BYTE;
  (* Bit stream of font *)

VAR
 font_id: STRING;
 info: InfoRec;
 y0, x0: INTEGER;
 chain: INTEGER;
 fheader: ^FontHeader;
 findex: ^FontIndices;
 fstream: ^FontBitStream;

PROCEDURE usage;
BEGIN
 WriteLn('SHOWFONT V', Version, Country, '  Show GEOS font file');
 WriteLn('Copyright (c) 1995,1996 Jochen Metzinger ');
 WriteLn;
 WriteLn(short_usage);
 WriteLn;
 WriteLn('  filename    GEOS font file');
 HALT(1);
END; (* usage *)

PROCEDURE OutLine(x, y: INTEGER; p: WORD; b: BYTE; width: WORD);
 VAR c: BYTE;
BEGIN
 c := (fstream^[p] SHL b) AND $FF;
 WHILE width > 0 DO BEGIN
  IF c AND $80 <> 0 THEN
   PutPixel(x,y,CharColor);
  INC(x);
  INC(b);
  IF b > 7 THEN
   BEGIN
    b := 0;
    INC(p);
    c := fstream^[p];
   END
  ELSE
   c := (c SHL 1) AND $FF;
  DEC(width);
 END; (* while *)
END; (* OutLine *)

PROCEDURE OutChar(fsize: INTEGER; c: INTEGER);
 VAR pos: WORD; width: LongInt;
  row: BYTE;
BEGIN
 pos := findex^[c];
 width := (findex^[c+1] - pos) - 1;
 IF width < 0 THEN
  G_FATAL('font '+long2str(fsize)+' at "'+CHR(c)+'" negativ width');
 IF pos DIV 8 > fheader^.bss THEN
  G_FATAL('font '+long2str(fsize)+' at "'+CHR(c)+'" index out of range');
 IF x0 + width > VGA_MAXX THEN BEGIN
  y0 := y0 + fsize + 2;
  x0 := 0;
 END; (* if *)
 Rectangle(x0, y0, x0+width-1, y0+fsize-1);
 Line(x0, y0+fheader^.ul, x0+width-1, y0+fheader^.ul);
 FOR row := 0 TO fsize-1 DO
  OutLine(x0, y0+row, fheader^.bss*row+(pos DIV 8), pos MOD 8, width);
 x0 := x0 + width + 1;
END; (* OutChar *)

PROCEDURE OutFont(nr: INTEGER);
 VAR i: INTEGER;
BEGIN
 SetColor(NameColor);
 OutTextXY(0,y0+8,font_id+' '+long2str(nr));
 Inc(y0,12);
 SetColor(BoxColor);
 cvt_chain(nr);
 IF cvt_size(nr) < SizeOf(FontHeader) THEN
  G_FATAL('font '+long2str(nr)+' too short');
 fheader := POINTER(Buffer);
 WITH fheader^ DO BEGIN
  IF pnt <> nr THEN
   G_FATAL('font '+long2str(nr)+' has wrong chain');
  IF ul > pnt THEN
   G_FATAL('font '+long2str(nr)+' underline error');
  IF SizeOf(fontHeader) > idx THEN
   G_FATAL('font '+long2str(nr)+' header & index crash');
  IF idx+SizeOf(FontIndices) > bs THEN
   G_FATAL('font '+long2str(nr)+' index & stream crash');
  IF bs + pnt*bss > cvt_size(nr) THEN
   G_FATAL('font '+long2str(nr)+' stream incomplete');
  findex  := Addr(Buffer^[idx]);
  fstream := Addr(Buffer^[bs]);
 END; (* with *)
 x0 := 0;
 FOR i := 32 TO 126 DO
  OutChar(nr,i);
 x0 := 0;
 y0 := y0 + nr + 12;
END; (* OutFont *)

BEGIN
 short_usage := 'SHOWFONT [/?] filename';
 IF (ParamStr(1) = '/?') OR (ParamStr(1) = '') THEN usage;
 cvt_open(ParamStr(1));
 font_id := cvt_name;
 cvt_info(info);
 IF info.gtyp <> 8 THEN FATAL('not a FONT');
 font_id := font_id + '['+long2str(info.w[128 DIV 2])+']';
 OpenGraphic(EGAcp);
 y0 := 0;
 FOR chain := 0 TO 126 DO
  IF cvt_chain_used(chain) THEN
   OutFont(chain);
 CloseGraphic(TRUE);
 cvt_close;
END. (* ShowFont *)
