(* TEXTS.PAS -- translate GEOS text pages
** Copyright (C) 1995,1996  Jochen Metzinger
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

UNIT Texts;

INTERFACE

USES global, fonts, cvt;

TYPE
 txt_style = (s_none, s_write, s_scrap, s_album, s_note);

VAR
 style: txt_style;
 max_chain: ShortInt;
 verbose: BOOLEAN;
 out_file: TEXT;

PROCEDURE OpenTxt;
(* get text style of an open .cvt file *)
PROCEDURE CloseTxt;
(* close text *)
PROCEDURE OutputChain(chain: BYTE);
(* Output page in chain *)

IMPLEMENTATION

CONST
 StyleDocName: ARRAY [txt_style] OF STRING =
  ('File', 'Write Image', 'Text Scrap', 'Text Album', 'Notes');
 StyleMaxChain: ARRAY [txt_style] OF ShortInt =
  (-1, 60, 0, 126, 126);

FUNCTION GetStyle: txt_style;
 VAR info: InfoRec;
BEGIN
 cvt_info(info);
 WITH info DO BEGIN
  IF ftyp AND $07 <> $03 THEN
   FATAL('not a USR file');
  IF gtyp = $07 THEN
   BEGIN (* doc file *)
    IF strk <> 1 THEN FATAL('not a VLIR file');
    IF GetString(class,15) = 'Write Image V1.' THEN
     GetStyle := s_write
    ELSE IF GetString(class,15) = 'text album  V1.' THEN
     GetStyle := s_album
    ELSE
     FATAL('not a text file');
   END
  ELSE IF gtyp = $04 THEN
   BEGIN (* sys file *)
    IF strk = 0 THEN
     (* scrap? *)
     IF GetString(class,15) = 'Text  Scrap V1.'
       THEN GetStyle := s_scrap
       ELSE FATAL('not a text file')
    ELSE IF strk = 1 THEN
     (* notes? *)
     IF GetString(class,15) = 'Notes       V1.'
      THEN GetStyle := s_note
      ELSE FATAL('not a text file')
    ELSE (* strk > 1 *)
     FATAL('neither SEQ nor VLIR file');
   END
  ELSE
   FATAL('neither DOC nor SYS file');
 END; (* with *)
END; (* GetStyle *)

VAR
 tab: ARRAY [0..7] OF WORD;
 nr_tab: 0..8;

PROCEDURE StandardTabs;
 VAR i: BYTE;
BEGIN
 nr_tab := 8;
 FOR i := 1 TO 8 DO
  tab[i-1] := 8*i;
END; (* StandardTabs *)

PROCEDURE NormalizeTabs(lft, rt: WORD);
 VAR i, len: WORD;
BEGIN
 IF lft >= rt THEN BEGIN
  StandardTabs;
  EXIT;
 END; (* if *)
 len := rt - lft + 1;
 FOR i := 0 TO nr_tab-1 DO BEGIN
  IF (tab[i] <= lft) OR (tab[i] >= rt) THEN BEGIN
   nr_tab := i;
   EXIT;
  END; (* if *)
  IF i > 0 THEN
   IF tab[i-1] >= tab[i] THEN BEGIN
    nr_tab := i;
    EXIT;
   END; (* if *)
  tab[i] := 66*(tab[i]-lft) DIV len;
 END; (* for *)
END; (* NormalizeTabs *)

VAR line: STRING;
    break: BOOLEAN;

PROCEDURE OutputLine;
BEGIN
 WriteLn(out_file, line);
 line := '';
END; (* OutputLine *)

PROCEDURE BreakLine;
BEGIN
 IF line <> '' THEN BEGIN
  line := line + '|';
  OutputLine;
  break := TRUE;
 END; (* if *)
END; (* BreakLine *)

PROCEDURE AddLine(s: STRING);
BEGIN
 IF (Length(line) > 60) AND (s = ' ') THEN BEGIN
  OutputLine;
  break := TRUE;
  EXIT;
 END; (* if *)
 line := line + s;
 break := FALSE;
END; (* AddLine *)

PROCEDURE LastLine;
BEGIN
 IF Length(line) >= 3 THEN
  IF Copy(line, Length(line)-2, 3) = '^^@' THEN
   BEGIN (* page break *)
    Delete(line, Length(line)-2, 3);
    IF line <> '' THEN OutputLine;
    break := FALSE;
   END; (* if *)
END; (* LastLine *)

PROCEDURE OpenTxt;
BEGIN
 style := GetStyle;
 max_chain := StyleMaxChain[style];
 nr_tab := 0;
 line := '';
 break := FALSE;
 IF verbose THEN
  WriteLn(out_file, '### ', StyleDocName[style], ': ', cvt_name);
END; (* OpenTxt *)

PROCEDURE PageHeader;
 VAR header: RECORD lft, rt: WORD; t: ARRAY [0..7] OF WORD; END;
  res: WORD; i: INTEGER; len: LongInt;
BEGIN
 cvt_read(header,SizeOf(header),res);
 IF res <> SizeOf(header) THEN
  FATAL('no write image header');
 WITH header DO BEGIN
  IF verbose THEN BEGIN
   BreakLine;
   Write(out_file, '### [ ', lft, ' | ');
   FOR i := 0 TO 7 DO Write(out_file, t[i], ' ');
   WriteLn(out_file, '| ', rt, ' ]');
  END; (* if *)
  FOR i := 0 TO 7 DO
   tab[i] := t[i];
  nr_tab := 8;
  NormalizeTabs(lft,rt);
 END; (* with *)
END; (* PageHeader *)

PROCEDURE ScrapHeader(chain: BYTE);
 VAR h_size, res: WORD;
BEGIN
 cvt_read(h_size, SizeOf(h_size), res);
 IF res <> SizeOf(h_size) THEN
  FATAL('no scrap header');
 INC(h_size, 2);
 IF cvt_size(chain) <> h_size THEN BEGIN
  IF verbose THEN BEGIN
   BreakLine;
   WriteLn(out_file, '### ', cvt_size(chain) , ' <> ', h_size);
  END; (* if *)
  cvt_resize(h_size);
 END; (* if *)
 StandardTabs;
END; (* ScrapHeader *)

PROCEDURE OutTab;
 VAR l, i, j: WORD;
BEGIN (* tab *)
 l := Length(line)+1;
 break := FALSE;
 FOR i := 0 TO nr_tab-1 DO
  IF l <= tab[i] THEN BEGIN
   FOR j := l TO tab[i] DO
    line := line + ' ';
   EXIT;
  END; (* if *)
 line := line + ' ';
END; (* OutTab *)

PROCEDURE OutGrph;
 VAR pic: RECORD w: BYTE; h: WORD; c: BYTE; END;
  res: WORD;
BEGIN
 cvt_read(pic,SizeOf(pic),res);
 IF res <> SizeOf(pic) THEN EXIT;
 WITH pic DO BEGIN
  BreakLine;
  WriteLn(out_file, '### photo scrap ',8*w,'x',h,' at $',HexStr(c,2));
 END; (* with *)
END; (* OutGrph *)

PROCEDURE OutNCS;
 VAR font: RECORD is: WORD; attr: BYTE; END;
  id, res, i: WORD;
BEGIN
 cvt_read(font,SizeOf(font),res);
 IF res <> SizeOf(font) THEN EXIT;
 id := font.is SHR 6;
 SelectFont(id);
 IF verbose THEN BEGIN
  BreakLine;
  Write(out_file, '### NEWCARDSET ',FontName,'[',font.is AND $3F,']');
  IF font.attr AND $80 <> 0 THEN Write(out_file,' underline');
  IF font.attr AND $40 <> 0 THEN Write(out_file,' bold');
  IF font.attr AND $10 <> 0 THEN Write(out_file,' italics');
  IF font.attr AND $08 <> 0 THEN Write(out_file,' outline');
  WriteLn(out_file);
 END; (* verbose *)
END; (* OutNCS *)

PROCEDURE OutPage;
 VAR ch: CHAR;
BEGIN
 WHILE cvt_getch(ch) DO BEGIN
  CASE ch OF
   #$09: OutTab;
   #$0D: IF break OR (line <> '') THEN BEGIN
          OutputLine;
          break := FALSE;
         END;
   #$10: OutGrph;
   #$17: OutNCS;
   ELSE  AddLine(TransChr(ch));
  END; (* case *)
 END; (* while *)
END; (* OutPage *)

PROCEDURE OutputPage(chain: BYTE);
BEGIN
 IF verbose THEN BEGIN
  BreakLine;
  WriteLn(out_file, '### page ',chain+1);
 END; (* if *)
 PageHeader;
 OutPage;
 LastLine;
END; (* BeginPage *)

PROCEDURE OutputScrap(chain: BYTE);
BEGIN
 IF style = s_album THEN BEGIN
  BreakLine;
  WriteLn(out_file, '### scrap ', chain + 1);
 END; (* if *)
 ScrapHeader(chain);
 OutPage;
 LastLine;
 IF line <> '' THEN OutputLine;
END; (* BeginScrap *)

PROCEDURE OutputNote(chain: BYTE);
 VAR ch: CHAR;
BEGIN
 WriteLn(out_file, '### note ', chain + 1);
 WHILE cvt_getch(ch) DO
  IF (ch <> #13) AND (ch <> #0) THEN
   line := line + ch
  ELSE BEGIN
   WriteLn(out_file, TransStr(line));
   line := '';
   IF ch = #0 THEN EXIT;
  END; (* else *)
 IF line <> '' THEN BEGIN
  WriteLn(out_file, TransStr(line),'|');
  line := '';
 END; (* if *)
END; (* OutputNote *)

PROCEDURE OutputChain(chain: BYTE);
BEGIN
 IF NOT cvt_chain_used(chain) THEN EXIT;
 cvt_chain(chain);
 CASE style OF
  s_write: OutputPage(chain);
  s_scrap,
  s_album: OutputScrap(chain);
  s_note:  OutputNote(chain);
 END; (* case *)
END; (* OutputChain *)

PROCEDURE CloseTxt;
BEGIN
 BreakLine;
 IF verbose THEN
  WriteLn(out_file, '### End Of ',StyleDocName[style]);
END; (* CloseTxt *)

BEGIN
 style := s_none;
 max_chain := -1;
 verbose := TRUE;
 nr_tab := 0;
 line := '';
 break := FALSE;
END. (* Texts *)